/**
 * @brief Interface to the AVR ADC hardware.
 * @details This file is...
 * @author Jake G
 * @date 2024
 * @copyright None
 * @file ADC.h
 */

#ifndef ADC_H
#define ADC_H

#include <stdbool.h>
#include <stdint.h>

/**
 * @brief The number of loop cycles it waits for ADC conversion before
 * timing out.
 */
#define ADC_WAIT_TIMEOUT 16

/**
 * @brief Initializes the AVR hardware in order to accept
 * Input for ADC usage.
 * @param adc_chan The channel of the pin 0-7 you are initializing.
 *
 * This function only makes use of PORTA by default. It sets the direction
 * register to input, disables the pull-up resistor and also diables interrupts
 * alongside the input buffer(digital).
 *
 * This in turn helps reduce noise when using the ADC.
 *
 */
void ADC_Init(uint8_t adc_chan);

/**
 * @brief Returns the pin number for the adc channel or 255 on an error.
 * @param The ADC channel.
 */
uint8_t ADC_GetChannelPinNum(uint8_t adc_chan);

/**
 * @brief Enables the ADC
 */
void ADC_Enable(void);

/**
 * @brief Disables the ADC
 */
void ADC_Disable();

/**
 * @brief Reads ADC value into variable
 *
 *
 * This function depends on the ADC already being initialized and enabled
 * before being called.
 */
extern uint16_t (*ADC_ReadValue)();

/**
 * @brief Sets up the ADC
 *
 * This function sets up the ADC to take and accumulate 32 samples. It also
 * sets the inital delay to 32 ADC clock cycles, and sets the VREF to VDD or
 * VCC.
 *
 * This function should only need to be called once.
 */
void ADC_Setup(void);

#endif // ADC_H
